/*
 
 *************************************************************************
 
 Armagetron Printer Interface
 Developed by Morgan Jones (maclover201@me.com)
 
 **************************************************************************
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 ***************************************************************************
 
 */

#include <iostream>
#include <sstream>
#include <ctime>
#include <deque>
#include <fstream>
#include <math.h>
#include <stdlib.h>
#include <signal.h>

#include "main.h"

#include "socket.h"
#include "player.h"
#include "printer.h"
#include "config.h"
#include "log.h"

static string ip( "0.0.0.0" );
static ConfigItem <string> pip( "PRINTER_IP", ip );

static string server_names( "NONE" );
static ConfigItem <string> sn( "SERVER_NAMES", server_names );

static string player_files( "NONE" );
static ConfigItem <string> pf( "PLAYER_FILES", player_files );

int display_max = 16;
static ConfigItem <int> dm( "DISPLAY_MAX", display_max );

int scroll_type = 0;
static ConfigItem <int> st( "SCROLL_TYPE", scroll_type );

static bool debug = false;
static ConfigItem <bool> dbg( "DEBUG", debug );

REAL scroll_rate = 0.2f;
static ConfigItem <REAL> sr( "SCROLL_RATE", scroll_rate );

REAL initial_pause = 1.5f;
static ConfigItem <REAL> inip( "INITIAL_PAUSE", initial_pause );

REAL intermediate_pause = 1.75f;
static ConfigItem <REAL> intp( "INTERMEDIATE_PAUSE", intermediate_pause );

REAL final_pause = 3.0f;
static ConfigItem <REAL> fp( "FINAL_PAUSE", final_pause );

static REAL intro_pause = 2.0f;
static ConfigItem <REAL> inp( "INTRO_PAUSE", intro_pause );

vector <Player> Main::Parse(vector <string> data)
{
	int size = data.size();
	
	if( size >= 2 )
	{
		data.pop_back();
		size--;
	}
	
	// do some preliminary parsing
	if( size > 1 ) // more than one line OF ACTUAL DATA?
	{
		data.erase( data.begin() ); // removes the headers
	} 
	
	if( size <= 1 && data[0].find( " online in spectator mode." ) != string::npos )
	{
		data[0] = "Nobody online.";
	}
	
	vector <Player> players;
	
	for( vector <string>::iterator it = data.begin(); it != data.end(); it++ )
	{
		Player temp = Player( *it );
		if( temp.Good() && !temp.Absent() )
			players.push_back( temp );
	}
	return players;
}

vector <string> Main::LoadFile(string path)
{
	ifstream file;
	
	file.open( path.c_str(), ifstream::in );
	
	if( !file.is_open() || !file.good() )
	{
		stringstream err;
		err << "Error opening file \"" << path << ".\"" << endl;
		Printer::Reset();
		Log::die( err.str(), 5 );
	}
	
	vector <string> contents;
	
	while( !file.eof() && file.good() )
	{
		string temp;
		getline( file, temp );
		contents.push_back(temp);
	}
	file.close();
	return contents;
}

vector <string> Main::Explode(string delimiter, string data)
{
	size_t start, end;
	vector <string> temp;
	
	start = 0;
	end = data.find( delimiter, start + 1 );
	if( end != string::npos )
	{
		temp.push_back( data.substr( start, end ) );
		while( start != string::npos && end != string::npos )
		{
			start = data.find( delimiter, start + 1 ) + delimiter.length();
			end = data.find( delimiter, start + 1 );
			temp.push_back( data.substr( start, end - start ) );
		}
	}
	return temp;
}

void Main::QuitProc(int worthlessParam)
{
	signal( SIGINT, Main::QuitProc );
	
	Printer::Reset();
	Socket::Destroy();
	
	stringstream information;
	information << "CTRL+C pressed. Quitting (" << worthlessParam << ")." << endl;
	Log::die( information.str(), EXIT_SUCCESS );
}

void Main::Loop(vector <string> player_files, vector <string> server_names)
{
	Socket::Make( ip );
	if( player_files.empty() || server_names.empty() || player_files[0] == "NONE" || server_names[0] == "NONE" )
		Log::die( "Please set your player file or server name entries in settings.cfg.\n", 0 );
	
	signal( SIGINT, Main::QuitProc );
	
	Printer::Reset();
	
	Printer::Display( "Armagetron" );
	wait( intro_pause );
	Printer::Display( "Printer" );
	wait( intro_pause );
	Printer::Display( "Interface" );
	wait( intro_pause );
	
	stringstream build;
	build << "Build " << VERSION;
	Printer::Display( build.str() );
	wait( intro_pause * 2.15 );
	
	// Infinite loop, abort with CTRL+C
	while( 1 )
	{
		for( int a = 0; a < 3; a++ )
		{
			time_t rawtime;
			struct tm * timeinfo;
			char buffer[display_max + 1];
			time ( &rawtime );
			timeinfo = localtime ( &rawtime );
			
			strftime ( buffer, display_max, "%b %d %X" , timeinfo ); 
			Printer::Display( buffer );
			
			wait( 3 ); // let the time increase by 3 seconds before forming the string again
		}
		
		for( int a = 0; a < player_files.size() && a < server_names.size(); a++ )
		{
			Printer::Display( server_names[a] );
			wait( final_pause );
			vector <Player> players = Main::Parse( Main::LoadFile( player_files[a] ) );
			
			if( players.size() > 0 )
			{
				for( int b = 0; b < players.size(); b++ )
				{
					stringstream toDisplay;
					
					if( players[b].Team() == "AI team" )
						toDisplay << "AI: ";
					toDisplay << players[b].Name() << " (";
					
					if( players[b].Alive() )
						toDisplay << "Alive";
					else
						toDisplay << "Dead";
					toDisplay << ')';
					Printer::Display( toDisplay.str() );
					wait( initial_pause );
					toDisplay.str( "" );
					
					toDisplay << "Score: " << players[b].Score();
					Printer::Display( toDisplay.str() );
					wait( initial_pause );
					toDisplay.str( "" );
					
					toDisplay << "Ping: " << players[b].Ping();
					Printer::Display( toDisplay.str() );
					wait( initial_pause );
					toDisplay.str( "" );
					
					if( players[b].Name() != players[b].Team() && players[b].Team() != "AI team" && !players[b].Team().empty() )
					{
						toDisplay << "Team: " << players[b].Team();
						Printer::Display( toDisplay.str() );
						wait( initial_pause );
					}
				}
			}
			else
			{
				Printer::Display( "Nobody online." );
				wait( initial_pause );
			}
		}
	}
}

int main(int argc, char * const argv[]) {
	Main::Loop( Main::Explode( ";", player_files ), Main::Explode( ";", server_names ) );
    return EXIT_SUCCESS;
}