/*
 
 *************************************************************************
 
 Armagetron Printer Interface
 Developed by Morgan Jones (maclover201@me.com)
 
 **************************************************************************
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 ***************************************************************************
 
 */

#include <iostream>
#include <string.h>
#include <sstream>
#include <stdlib.h>
#include <netdb.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/select.h>
#include <netinet/tcp.h>
#include <netinet/ip.h>
#include <netinet/in.h>

#include "log.h"
#include "config.h"
#include "socket.h"

static bool no_network = false;
static ConfigItem <bool> nn( "NO_NETWORK", no_network );

// Both from main.cpp
extern int display_max;
extern int scroll_type;

int Socket::sockfd;

void Socket::Write(string message, bool printerFormat)
{
	if( printerFormat )
	{
		int bytes_sent = 0;
		if( !no_network )
		{
			stringstream d;
			d << "\033%-12345X@PJL RDYMSG DISPLAY = \"" << message << "\"\r\n\033%-12345X\r\n";
			if ( bytes_sent = send( sockfd, d.str().c_str(), d.str().length(), NULL ) < 0 ) 
			{
				stringstream err;
				err << "Error writing to socket." << endl;
				Log::die( err.str(), ERR_WRITING );
			}
		}
		if( scroll_type == 0 )
		{
			if( message.find( "\xFD" ) != string::npos )
				message.replace( message.find( "\xFD" ), message.length(), ">" ); // clean up the 0xFD for COUT
		}
		if( !message.empty() )
		{
			cout << '[' << message;
			for( int a = 0; a < display_max - message.length(); a++ )
				cout << ' ';
			cout << ']';
			if( !no_network && bytes_sent > 0 )
				cout << " (" << bytes_sent << " bytes)";
			cout << endl;
		}
	}
	else
	{
		int bytes_sent = 0;
		if ( bytes_sent = send( sockfd, message.c_str(), message.length(), NULL ) < 0 ) 
		{
			stringstream err;
			err << "Error writing to socket." << endl;
			Log::die( err.str(), ERR_WRITING );
		}
		if( !message.empty() )
		{
			cout << '[' << message;
			for( int a = 0; a < display_max - message.length(); a++ )
				cout << ' ';
			cout << ']';
			if( !no_network )
				cout << " (" << bytes_sent << " bytes)";
			cout << endl;
		}
	}
}

int Socket::Connect(struct sockaddr *addr, int addrlen, struct timeval *timeout)
{
	struct timeval sv;
	socklen_t svlen = sizeof( sv );
	int ret;
	
	// Try setting TOS to LOWDELAY for faster transfers. It's not critical but helps.
	char tos = IPTOS_LOWDELAY;
	ret = setsockopt( sockfd, IPPROTO_IP, IP_TOS, &tos, sizeof( char ) );
	
	if( ret != 0 )
	{
		stringstream warning;
		warning << "Setting TOS to LOWDELAY failed." << endl;
		Log::warn( warning.str() );
	}
	
	// Try turning off TCP delay for a realtime response.
	int tcpnodelay_flag = 1;
	ret = setsockopt( sockfd, IPPROTO_TCP, TCP_NODELAY, (char *) &tcpnodelay_flag, sizeof( int ) );
	
	if( ret != 0 )
	{
		stringstream warning;
		warning << "Could not turn off TCP delay." << endl;
		Log::warn( warning.str() );
	}
	
	// If timeout is NULL for some reason, simply connect.
	if( !timeout )
	{
		ret = connect( sockfd, addr, addrlen );
		return ret;
	}
	
	// Try getting the current timeout.
	ret = getsockopt( sockfd, SOL_SOCKET, SO_RCVTIMEO, (char *)&sv, &svlen );
	if( ret < 0 )
		return -1;
	
	// Try setting the new timeout.
	ret = setsockopt( sockfd, SOL_SOCKET, SO_RCVTIMEO, (char *)timeout, sizeof( *timeout ) );
	if( ret < 0 )
		return -1;
	
	// Connect.
	ret = connect( sockfd, addr, addrlen );
	
	// Restore the original timeout.
	setsockopt( sockfd, SOL_SOCKET, SO_RCVTIMEO, (char *)&sv, sizeof( sv ) );
	return ret;
}

void Socket::Make(string host)
{
	if( no_network )
	{
		sockfd = -1;
		return;
	}
	
	if( host == "0.0.0.0" )
		Log::die( "Please set PRINTER_IP to the IP of your HP printer in settings.cfg.\n", ERR_OPENING );
	
	int port = 9100; // standard port
	struct sockaddr_in serv_addr;
	struct hostent *server;
	
	// This fixed another one of those nasty memory leaks... don't point to a timeval
	struct timeval timeout;
	
	// Set the socket timeout to 10 seconds.
	timeout.tv_sec = 10;
	timeout.tv_usec = 0;
	
	sockfd = socket( AF_INET, SOCK_STREAM, 0 );
	
	if ( sockfd < 0 )
	{
		stringstream err;
		err << "Error opening socket to \"" << host << ".\"" << endl;
		Log::die( err.str(), ERR_OPENING );
	}
	
	server = gethostbyname( host.c_str() );
	
	if ( server == NULL )
	{
		stringstream err;
		err << "Host " << host << " does not exist." << endl;
		Log::die( err.str(), ERR_NOEXIST );
	}
	
	memset( (char *) &serv_addr, 0, sizeof( serv_addr ) );
	serv_addr.sin_family = AF_INET;
	memcpy( (char *) &serv_addr.sin_addr.s_addr, (char *) server->h_addr, server->h_length );
	serv_addr.sin_port = htons( port );
	
	cerr << "Connecting to host..." << endl;
	
	if ( Connect( (sockaddr*)&serv_addr, sizeof( serv_addr ), &timeout ) < 0 )
	{
		cerr << endl;
		stringstream err;
		err << "Error connecting to host " << host << "." << endl;
		Log::die( err.str(), ERR_CONNECT );
	}
}

void Socket::Destroy()
{
	if( close( sockfd ) != 0 && errno == EINTR )
	{
		stringstream information;
		information << "Error closing socket " << sockfd << "; it doesn't matter anyway as we're exiting." << endl;
		Log::info( information.str() );
	}
}